package main

import (
	"github.com/sony/sonyflake"
	"strconv"
	"strings"
	"time"
)

func main() {
	startTime := time.Date(2018, 9, 15, 7, 59, 59, 0, time.UTC)
	keyGen := sonyflake.NewSonyflake(sonyflake.Settings{
		StartTime: startTime,
		MachineID: func() (uint16, error) {
			return 1, nil
		},
	})

	n := 3
	for i := 0; i < 500; i++ {
		newId, _ := keyGen.NextID()

		idStr := strconv.FormatUint(newId, 10)
		sLen := len(idStr)
		subLen := sLen / n
		if sLen%n != 0 {
			subLen++
		}

		var partArr []string
		var codeArr []string
		for j := 0; j < n; j++ {
			start := subLen * j
			end := subLen * (j + 1)
			if end >= sLen {
				end = sLen
			}
			part := idStr[start:end]
			partArr = append(partArr, part)

			partNum, _ := strconv.ParseInt(part, 10, 32)
			code := decimalToAny(int(partNum), 35)
			codeArr = append(codeArr, code)
		}

		println(newId, " : ", strings.Join(partArr, "-"), " : ", strings.Join(codeArr, "-"))
	}
}

var tenToAny = map[int]string{0: "0", 1: "1", 2: "2", 3: "3", 4: "4", 5: "5", 6: "6", 7: "7", 8: "8", 9: "9", 10: "a", 11: "b", 12: "c", 13: "d", 14: "e", 15: "f", 16: "g", 17: "h", 18: "i", 19: "j", 20: "k", 21: "l", 22: "m", 23: "n", 24: "o", 25: "p", 26: "q", 27: "r", 28: "s", 29: "t", 30: "u", 31: "v", 32: "w", 33: "x", 34: "y", 35: "z", 36: ":", 37: ";", 38: "<", 39: "=", 40: ">", 41: "?", 42: "@", 43: "[", 44: "]", 45: "^", 46: "_", 47: "{", 48: "|", 49: "}", 50: "A", 51: "B", 52: "C", 53: "D", 54: "E", 55: "F", 56: "G", 57: "H", 58: "I", 59: "J", 60: "K", 61: "L", 62: "M", 63: "N", 64: "O", 65: "P", 66: "Q", 67: "R", 68: "S", 69: "T", 70: "U", 71: "V", 72: "W", 73: "X", 74: "Y", 75: "Z"}

func decimalToAny(num int, n int) string {
	new_num_str := ""
	var remainder int
	var remainder_string string
	for num != 0 {
		remainder = num % n
		if 76 > remainder && remainder > 9 {
			remainder_string = tenToAny[remainder]
		} else {
			remainder_string = strconv.Itoa(remainder)
		}
		new_num_str = remainder_string + new_num_str
		num = num / n
	}
	return new_num_str
}
